#!/usr/bin/env python3

DEFAULT_PDF_WHAT = 'the content of a PDF file'
DEFAULT_URL_WHAT = 'the content of a webpage'
DEFAULT_HTML_WHAT = 'the text content of a html file'
DEFAULT_GENERAL_WHAT = 'the content of a document'
